# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'range_widget.ui'
#
# Created by: PyQt5 UI code generator 5.15.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_RangeWidget(object):
    def setupUi(self, RangeWidget):
        RangeWidget.setObjectName("RangeWidget")
        RangeWidget.resize(673, 41)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(RangeWidget.sizePolicy().hasHeightForWidth())
        RangeWidget.setSizePolicy(sizePolicy)
        RangeWidget.setMinimumSize(QtCore.QSize(0, 41))
        RangeWidget.setMaximumSize(QtCore.QSize(16777215, 41))
        self.horizontalLayout = QtWidgets.QHBoxLayout(RangeWidget)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(RangeWidget)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.value1 = QtWidgets.QLineEdit(RangeWidget)
        self.value1.setObjectName("value1")
        self.horizontalLayout.addWidget(self.value1)
        self.op1 = QtWidgets.QComboBox(RangeWidget)
        self.op1.setEnabled(False)
        self.op1.setObjectName("op1")
        self.op1.addItem("")
        self.op1.addItem("")
        self.op1.addItem("")
        self.op1.addItem("")
        self.op1.addItem("")
        self.op1.addItem("")
        self.horizontalLayout.addWidget(self.op1)
        self.name = QtWidgets.QLabel(RangeWidget)
        self.name.setAlignment(QtCore.Qt.AlignCenter)
        self.name.setObjectName("name")
        self.horizontalLayout.addWidget(self.name)
        self.op2 = QtWidgets.QComboBox(RangeWidget)
        self.op2.setEnabled(False)
        self.op2.setObjectName("op2")
        self.op2.addItem("")
        self.op2.addItem("")
        self.op2.addItem("")
        self.op2.addItem("")
        self.op2.addItem("")
        self.op2.addItem("")
        self.horizontalLayout.addWidget(self.op2)
        self.value2 = QtWidgets.QLineEdit(RangeWidget)
        self.value2.setObjectName("value2")
        self.horizontalLayout.addWidget(self.value2)
        self.label_2 = QtWidgets.QLabel(RangeWidget)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout.addWidget(self.label_2)
        self.background_color_btn = QtWidgets.QPushButton(RangeWidget)
        self.background_color_btn.setMinimumSize(QtCore.QSize(40, 0))
        self.background_color_btn.setText("")
        self.background_color_btn.setObjectName("background_color_btn")
        self.horizontalLayout.addWidget(self.background_color_btn)
        self.label_3 = QtWidgets.QLabel(RangeWidget)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout.addWidget(self.label_3)
        self.font_color_btn = QtWidgets.QPushButton(RangeWidget)
        self.font_color_btn.setMinimumSize(QtCore.QSize(40, 0))
        self.font_color_btn.setText("")
        self.font_color_btn.setObjectName("font_color_btn")
        self.horizontalLayout.addWidget(self.font_color_btn)
        self.horizontalLayout.setStretch(3, 1)

        self.retranslateUi(RangeWidget)
        QtCore.QMetaObject.connectSlotsByName(RangeWidget)

    def retranslateUi(self, RangeWidget):
        _translate = QtCore.QCoreApplication.translate
        RangeWidget.setWindowTitle(_translate("RangeWidget", "Form"))
        self.label.setText(_translate("RangeWidget", "If"))
        self.op1.setItemText(0, _translate("RangeWidget", "=="))
        self.op1.setItemText(1, _translate("RangeWidget", "!="))
        self.op1.setItemText(2, _translate("RangeWidget", ">"))
        self.op1.setItemText(3, _translate("RangeWidget", ">="))
        self.op1.setItemText(4, _translate("RangeWidget", "<"))
        self.op1.setItemText(5, _translate("RangeWidget", "<="))
        self.name.setText(_translate("RangeWidget", "TextLabel"))
        self.op2.setItemText(0, _translate("RangeWidget", "=="))
        self.op2.setItemText(1, _translate("RangeWidget", "!="))
        self.op2.setItemText(2, _translate("RangeWidget", ">"))
        self.op2.setItemText(3, _translate("RangeWidget", ">="))
        self.op2.setItemText(4, _translate("RangeWidget", "<"))
        self.op2.setItemText(5, _translate("RangeWidget", "<="))
        self.label_2.setText(_translate("RangeWidget", "then set background"))
        self.label_3.setText(_translate("RangeWidget", "and font"))
