# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'fft_window.ui'
#
# Created by: PyQt5 UI code generator 5.15.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_FFTWindow(object):
    def setupUi(self, FFTWindow):
        FFTWindow.setObjectName("FFTWindow")
        FFTWindow.resize(640, 480)
        self.centralwidget = QtWidgets.QWidget(FFTWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.centralwidget)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.label_2 = QtWidgets.QLabel(self.centralwidget)
        self.label_2.setObjectName("label_2")
        self.verticalLayout.addWidget(self.label_2)
        self.start_frequency = QtWidgets.QDoubleSpinBox(self.centralwidget)
        self.start_frequency.setDecimals(3)
        self.start_frequency.setMinimum(0.001)
        self.start_frequency.setMaximum(1000000.0)
        self.start_frequency.setProperty("value", 1.0)
        self.start_frequency.setObjectName("start_frequency")
        self.verticalLayout.addWidget(self.start_frequency)
        self.label = QtWidgets.QLabel(self.centralwidget)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.end_frequency = QtWidgets.QDoubleSpinBox(self.centralwidget)
        self.end_frequency.setDecimals(3)
        self.end_frequency.setMinimum(0.001)
        self.end_frequency.setMaximum(1000000.0)
        self.end_frequency.setProperty("value", 1000.0)
        self.end_frequency.setObjectName("end_frequency")
        self.verticalLayout.addWidget(self.end_frequency)
        self.label_3 = QtWidgets.QLabel(self.centralwidget)
        self.label_3.setObjectName("label_3")
        self.verticalLayout.addWidget(self.label_3)
        self.frequency_step = QtWidgets.QDoubleSpinBox(self.centralwidget)
        self.frequency_step.setDecimals(3)
        self.frequency_step.setMinimum(0.001)
        self.frequency_step.setMaximum(1000000.0)
        self.frequency_step.setProperty("value", 1.0)
        self.frequency_step.setObjectName("frequency_step")
        self.verticalLayout.addWidget(self.frequency_step)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.horizontalLayout.addLayout(self.verticalLayout)
        self.layout = QtWidgets.QVBoxLayout()
        self.layout.setObjectName("layout")
        self.horizontalLayout.addLayout(self.layout)
        self.horizontalLayout.setStretch(1, 1)
        FFTWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(FFTWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 640, 21))
        self.menubar.setObjectName("menubar")
        FFTWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(FFTWindow)
        self.statusbar.setObjectName("statusbar")
        FFTWindow.setStatusBar(self.statusbar)

        self.retranslateUi(FFTWindow)
        QtCore.QMetaObject.connectSlotsByName(FFTWindow)

    def retranslateUi(self, FFTWindow):
        _translate = QtCore.QCoreApplication.translate
        FFTWindow.setWindowTitle(_translate("FFTWindow", "MainWindow"))
        self.label_2.setText(_translate("FFTWindow", "Start frequency"))
        self.start_frequency.setSuffix(_translate("FFTWindow", "Hz"))
        self.label.setText(_translate("FFTWindow", "End frequency"))
        self.end_frequency.setSuffix(_translate("FFTWindow", "Hz"))
        self.label_3.setText(_translate("FFTWindow", "Frequency step"))
        self.frequency_step.setSuffix(_translate("FFTWindow", "Hz"))
