# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'gps_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_GPSDialog(object):
    def setupUi(self, GPSDialog):
        GPSDialog.setObjectName("GPSDialog")
        GPSDialog.resize(623, 190)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/globe.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        GPSDialog.setWindowIcon(icon)
        GPSDialog.setSizeGripEnabled(True)
        self.gridLayout = QtWidgets.QGridLayout(GPSDialog)
        self.gridLayout.setObjectName("gridLayout")
        self.label_3 = QtWidgets.QLabel(GPSDialog)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 0, 0, 1, 3)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.cancel_btn = QtWidgets.QPushButton(GPSDialog)
        self.cancel_btn.setObjectName("cancel_btn")
        self.horizontalLayout.addWidget(self.cancel_btn)
        spacerItem1 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.apply_btn = QtWidgets.QPushButton(GPSDialog)
        self.apply_btn.setObjectName("apply_btn")
        self.horizontalLayout.addWidget(self.apply_btn)
        self.horizontalLayout.setStretch(0, 1)
        self.gridLayout.addLayout(self.horizontalLayout, 5, 0, 1, 3)
        self.latitude = QtWidgets.QLineEdit(GPSDialog)
        self.latitude.setObjectName("latitude")
        self.gridLayout.addWidget(self.latitude, 2, 1, 1, 1)
        self.search_latitude_btn = QtWidgets.QPushButton(GPSDialog)
        self.search_latitude_btn.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/search.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.search_latitude_btn.setIcon(icon1)
        self.search_latitude_btn.setObjectName("search_latitude_btn")
        self.gridLayout.addWidget(self.search_latitude_btn, 2, 2, 1, 1)
        self.label_2 = QtWidgets.QLabel(GPSDialog)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 3, 0, 1, 1)
        self.label = QtWidgets.QLabel(GPSDialog)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 2, 0, 1, 1)
        self.search_longitude_btn = QtWidgets.QPushButton(GPSDialog)
        self.search_longitude_btn.setText("")
        self.search_longitude_btn.setIcon(icon1)
        self.search_longitude_btn.setObjectName("search_longitude_btn")
        self.gridLayout.addWidget(self.search_longitude_btn, 3, 2, 1, 1)
        spacerItem2 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem2, 1, 0, 1, 1)
        self.longitude = QtWidgets.QLineEdit(GPSDialog)
        self.longitude.setObjectName("longitude")
        self.gridLayout.addWidget(self.longitude, 3, 1, 1, 1)
        spacerItem3 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem3, 4, 0, 1, 1)
        self.gridLayout.setRowStretch(4, 1)

        self.retranslateUi(GPSDialog)
        QtCore.QMetaObject.connectSlotsByName(GPSDialog)

    def retranslateUi(self, GPSDialog):
        _translate = QtCore.QCoreApplication.translate
        GPSDialog.setWindowTitle(_translate("GPSDialog", "GPS channels selection"))
        self.label_3.setText(_translate("GPSDialog", "Search for the latitude and longitude channels:"))
        self.cancel_btn.setText(_translate("GPSDialog", "Cancel"))
        self.apply_btn.setText(_translate("GPSDialog", "Apply"))
        self.label_2.setText(_translate("GPSDialog", "Longitude"))
        self.label.setText(_translate("GPSDialog", "Latitude"))
from . import resource_rc
